#if !defined  HAVE_BIN2PAUL_H__
#define       HAVE_BIN2PAUL_H__

#include "fxttypes.h"

inline void bin2paul(ulong x, ulong &pp, ulong &pm)
// Compute M.Paul's signed binary representation of x:
// the unique representation of x as
//   x=\sum_{k}{d_k*2^k} where d_j \in {-1,0,+1}
//   and no two adjacent digits d_j, d_{j+1} are both nonzero.
// pp has bits j set where d_j==+1
// pm has bits j set where d_j==-1
// Thereby:  x = pp - pm
{
    ulong xh = x >> 1;  // x/2
    ulong x3 = x + xh;  // 3*x/2
    ulong c = xh ^ x3;
    pp = x3 & c;
    pm = xh & c;
}
// -------------------------


inline ulong paul2bin(ulong pp, ulong pm)
// Inverse of bin2paul()
{
    return ( pp - pm );
}
// -------------------------

#endif  // !defined HAVE_BIN2PAUL_H__
