#if !defined  HAVE_BITNECKLACE_H__
#define       HAVE_BITNECKLACE_H__


#include "fxttypes.h"
#include "bits/bitsperlong.h"
#include "bits/bitperiodic.h"  // bit_copy_periodic()
#include "bits/bithigh.h"  // highest_zero_idx()
#include "bits/tinyfactors.h"


class bit_necklace
// Binary necklaces as binary words.
// The cyclic maximal words are produced.
// Must have 0<n<=BITS_PER_LONG.
// cf. class binary_prime_string
{
public:
    ulong a_;  // necklace
    ulong n2_;  // bit representing n: n2==2**(n-1)
    ulong j2_;  // bit representing j: j2==2**(j-1)
    ulong n_;
    ulong j_;  // period of the necklace
    ulong mm_; // mask of n ones
    ulong tfb_; // for fast factor lookup

public:
    bit_necklace(ulong n)
    {
        if ( 0==n )  n = 1;  // avoid hang
        if ( n>=BITS_PER_LONG )  n = BITS_PER_LONG;
        init(n);
    }

    ~bit_necklace()  { ; }

    void init(ulong n)
    {
        a_ = 0;
        n_ = n;
        n2_ = 1UL<<(n-1);
        mm_ = (~0UL) >> (BITS_PER_LONG-n);
        tfb_ = tiny_factors_tab[n] >> 1;
        tfb_ |= n2_;  // needed for n==BITS_PER_LONG
        reset();
    }
    void reset()  { a_ = 0;  j2_ = 1;  j_ = 1; }

    ulong data() const { return  a_; }
    ulong period() const { return j_; }
    ulong is_lyndon_word()  const  { return (j2_ & n2_); }

    ulong next()  // return the period (zero when finished)
    {
    redo:
        if ( a_==mm_ )  { reset();  return 0; }

        j_ = highest_zero_idx( a_ ^ (~mm_)  ) + 1;
        j2_ = 1UL << (j_-1);

        a_ |= j2_;

//        a_ = bit_copy_periodic(a_, j_);
//        a_ &= mm_;
        a_ = bit_copy_periodic(a_, j_, n_);

        if ( 0==(tfb_ & j2_) )  goto redo;  // necklaces only

        return  j_;
    }

};
// -------------------------


#endif  // !defined HAVE_BITNECKLACE_H__
