#if !defined  HAVE_BITPOLSQUAREFREE_H__
#define       HAVE_BITPOLSQUAREFREE_H__

#include "bpol/bitpolderiv.h"
#include "bpol/bitpolgcd.h"
#include "bits/bitsperlong.h"
#include "fxttypes.h"  // ulong


inline ulong bitpol_test_squarefree(ulong c)
// Return  0 if polynomial is squarefree
// else return square factor != 0
{
    ulong d = bitpol_deriv(c);
    if ( 0==d )  return  (1==c ? 0 : c);
    ulong g = bitpol_gcd(c, d);
    return  (1==g ? 0 : g);
}
// -------------------------

inline ulong bitpol_pure_square_q(ulong c)
// Return whether polynomial is a pure square != 1
{
    if ( 1UL==c )  return 0;
#if  BITS_PER_LONG >= 64
    c &= 0xaaaaaaaaaaaaaaaaUL;
#else
    c &= 0xaaaaaaaaUL;
#endif
    return  (0==c);
}
// -------------------------

inline ulong bitpol_pure_sqrt(ulong c)
// Return t = sqrt(c)
// c must be a pure square.
{
    ulong t = 0;
    for (ulong mc=1,mt=1;  mc;  mc<<=2,mt<<=1)
    {
        if ( mc & c )  t |= mt;
    }
    return  t;
}
// -------------------------

// bpol/bitpolsquarefree.cc:
ulong bitpol_sreduce(ulong c);
ulong bitpol_factor_squarefree(ulong c, ulong *sf, ulong *se);


inline ulong bitpol_squarefree_part(ulong c)
// remove all factors with even exponents
{
    ulong g = bitpol_gcd(c, bitpol_deriv(c));
    if ( 1==g )  return c;
    else  return bitpol_div(c, g);
}
// -------------------------


inline ulong bitpol_make_squarefree(ulong c)
// reduce all exponents to one
{
    ulong z = c, t;
    while ( z!=(t=bitpol_sreduce(z)) )  z = t;
    return z;
}
// -------------------------



#endif  // !defined HAVE_BITPOLSQUAREFREE_H__
