#if !defined HAVE_COMBINATION_PREF_H__
#define      HAVE_COMBINATION_PREF_H__

#include "fxttypes.h"
//#include "fxtio.h"


class combination_pref
// (s,t)-combinations via prefix shifts ("cool-lex" order)
// corresponding to (n choose k) combinations where n=s+t, k=t.
//.
//  Algorithm as in
//  Frank Ruskey, Aaron Williams:
//    "Generating combinations by prefix shifts"
//    Lecture Notes in Computer Science, 3595, 2005.
//    Extended Abstract for COCOON 2005.
{
public:
    ulong *b_;
    ulong s_, t_, n_;
private:
    ulong x, y;

public:
    combination_pref(ulong s, ulong t)
    // Must have: s!=0, t!=0
    {
        s_ = s;
        t_ = t;
        n_ = s_ + t_;
        b_ = new ulong[n_];
        init();
    }

    ~combination_pref()
    {
        delete [] b_;
    }

    void init()
    {
        for (ulong j=0; j<n_; ++j)  b_[j] = 0;
        for (ulong j=0; j<t_; ++j)  b_[j] = 1;
        x = 0;  y = 0;
    }

    bool next()
    {
        if ( x==0 )  { x=1;  b_[t_]=1;  b_[0]=0;  return 1; }
        else
        {
            if ( x>=n_-1 )  return 0;
            else
            {
                b_[x++] = 0;  b_[y++] = 1;      // X(s,t)
                if ( b_[x]==0 )
                {
                    b_[x] = 1;  b_[0] = 0;      // Y(s,t)
                    if ( y>1 )  x = 1;          // Z(s,t)
                    y = 0;
                }
            }
        }
    }

    const ulong * data() const  { return b_; }
    ulong size() const  { return n_; }
};
// -------------------------


#endif  // !defined HAVE_COMBINATION_PREF_H__
