#if !defined HAVE_COMPOSITION_COLEX_H__
#define      HAVE_COMPOSITION_COLEX_H__


#include "fxttypes.h"


class composition_colex
// k-compositions of n, co-lexicographic (colex) order
{
public:
    ulong n_, k_;  // n is the sum of k elements
    ulong *x_;  // data

public:
    composition_colex(ulong n, ulong k)
    {
        n_ = (n ? n : 1);  // not zero
        k_ = (k ? k : 1);  // not zero
        x_ = new ulong[k_];
        first();
    }

    ~composition_colex()  { delete [] x_; }

    const ulong *data()  const  { return x_; }

    void first()
    {
        x_[0] = n_;
        for (ulong k=1; k<k_; ++k)  x_[k] = 0;
    }

    void last()
    {
        for (ulong k=0; k<k_; ++k)  x_[k] = 0;
        x_[k_-1] = n_;
    }

    ulong next()
    // Nijenhuis, Wilf
    {
        // Return zero if current comp. is last:
        if ( n_==x_[k_-1] )  return 0;

        ulong j = 0;
        while ( 0==x_[j] )  ++j;  // find first nonzero
        ulong v = x_[j];  // value of first nonzero
        x_[j] = 0;        // set to zero
        x_[0] = v - 1;    // value-1 to first position
        ++x_[j+1];        // increment next position

        return  1;
    }

    ulong prev()
    {
        // Return zero if current comp. is first:
        if ( n_==x_[0] )  return 0;

        ulong v0 = x_[0];   // vlaue at first position
        x_[0] = 0;          // set first position to zero
        ulong j = 1;
        while ( 0==x_[j] )  ++j;  // find next nonzero
        --x_[j];                  // decrement value
        x_[j-1] = 1 + v0;         // set previous position

        return  1;
    }
};
// -------------------------

#endif  // !defined HAVE_COMPOSITION_COLEX_H__
