#if !defined  HAVE_KSUBSET_NEAR_PERFECT_H__
#define       HAVE_KSUBSET_NEAR_PERFECT_H__


#include "aux0/swap.h"
#include "fxttypes.h"

class ksubset_near_perfect
// k-subsets (kmin<=k<=kmax) in near-perfect order.
// Recursive algorithm.
{
public:
    ulong n_; // k-subsets of the n-element set
    ulong kmin_, kmax_;   // k-subsets with kmin<=k<=kmax
    ulong *rv_;  // record of visits in graph (delta set)
    ulong ct_;   // count objects
    // ulong k_;    // number of elements in subset
    ulong rct_;  // count recursions (==work)
    ulong nq_;   // whether to reverse order
    // function to call with each combination:
    void (*visit_)(const ksubset_near_perfect &);

public:
    ksubset_near_perfect(ulong n)
    {
        n_ = n;
        rv_ = new ulong[n_+1];  ++rv_;
        rv_[-1] = 0;
    }


    ~ksubset_near_perfect()
    {
        --rv_;
        delete [] rv_;
    }

    void generate(void (*visit)(const ksubset_near_perfect &),
                  ulong kmin, ulong kmax, ulong nq=0)
    {
        ct_ = 0;
        rct_ = 0;

        kmin_ = kmin;
        kmax_ = kmax;
        if ( kmin_ > kmax_ )  swap2(kmin_, kmax_);
        if ( kmax_ > n_ )  kmax_ = n_;
        if ( kmin_ > n_ )  kmin_ = n_;

        visit_ = visit;
        nq_ = nq;
        next_rec(0, 0, 0);
    }

private:
    void next_rec(ulong d, ulong p, ulong e1);
};
// -------------------------



#endif  // !defined HAVE_KSUBSET_NEAR_PERFECT_H__
