#if !defined HAVE_MIXEDRADIX_GRAY_H__
#define      HAVE_MIXEDRADIX_GRAY_H__

#include "fxttypes.h"


class mixedradix_gray
// Gray code for mixed radix numbers.
// Loopless algorithm. Implementation following Knuth.
{
public:
    ulong *a_;  // digits
    ulong *m1_; // radix minus one ('nines')
    ulong *f_;  // focus pointer
    ulong *d_;  // direction
    ulong n_;  // number of digits
    ulong j_;  // aux pointer
    int dm_;  // direction of last move

public:
    mixedradix_gray(const ulong *m, ulong n, ulong mm=0)
    {
        n_ = n;
        a_ = new ulong[n_];
        m1_ = new ulong[n_];
        d_ = new ulong[n_];    // m1_[j] == m[j] - 1 in Knuth
        f_ = new ulong[n_+1];  // n_ + 1 elements

        // if mm given it is used as radix for all digits:
        if ( 0==mm )  for (ulong k=0; k<n_; ++k)  m1_[k] = m[k] - 1;
        else          for (ulong k=0; k<n_; ++k)  m1_[k] = mm - 1;

        first();
    }

    ~mixedradix_gray()
    {
        delete [] a_;
        delete [] m1_;
        delete [] d_;
        delete [] f_;
    }

    const ulong *data()  const  { return a_; }


    void first()
    {
        for (ulong k=0; k<n_; ++k)  a_[k] = 0;
        for (ulong k=0; k<n_; ++k)  d_[k] = 1;
        for (ulong k=0; k<=n_; ++k)  f_[k] = k;
        dm_ = 0;
        j_ = n_;
    }

    bool next()
    {
        ulong j = f_[0];
        f_[0] = 0;

        if ( j>=n_ )  { first(); return 0; }
        a_[j] += d_[j];
        dm_ = (int)d_[j];

        ulong aj = a_[j];
        if ( (0==aj) || (aj==m1_[j]) )
        {
            d_[j] = -d_[j];
            f_[j] = f_[j+1];
            f_[j+1] = j + 1;
        }
        j_ = j;
        return 1;
    }

    ulong pos()  const  { return j_; }  // position of last change
    int dir()  const  { return dm_; }
};
// -------------------------


#endif  // !defined HAVE_MIXEDRADIX_GRAY_H__
