#if !defined HAVE_MIXEDRADIX_GRAY2_H__
#define      HAVE_MIXEDRADIX_GRAY2_H__

#include "fxttypes.h"


class mixedradix_gray2
// Gray code for mixed radix numbers.
// CAT algorithm.
{
public:
    ulong *d_;  // mixed radix digits
    ulong *m1_; // radices (minus one)
    ulong *i_;  // direction
    ulong n_;   // permutations of n elements
    ulong j_;  // aux pointer
    int dm_;  // direction of last move

public:
    mixedradix_gray2(const ulong *m, ulong n, ulong mm=0)
    {
        n_ = n + 1;
        d_ = new ulong[n_];
        d_[n-1] = -1UL;  // sentinel
        i_ = new ulong[n_];
        i_[n_-1] = 0;
        m1_ = new ulong[n_];

        if ( 0==mm )  for (ulong k=0; k<n_; ++k)  m1_[k] = m[k] - 1;
        else          for (ulong k=0; k<n_; ++k)  m1_[k] = mm - 1;

        first();
    }

    ~mixedradix_gray2()
    {
        delete [] i_;
        delete [] d_;
        delete [] m1_;
    }

    const ulong *data()  const { return d_; }

    void first()
    {
        for (ulong k=0; k<n_-1; ++k)  d_[k] = 0;
        for (ulong k=0; k<n_-1; ++k)  i_[k] = +1;
    }

    void last()
    {
        first();
        d_[n_-2] = 1;
        for (ulong k=0; k<n_-2; ++k)  i_[k] = -1UL;
        i_[n_-2] = +1;
    }

    bool next()
    {
        // increment mixed-radix number:
        ulong j = 0;
        ulong ij;
        ulong im;
//        while ( j<n_-1 )
        while ( (ij=i_[j]) )
        {
            im = i_[j];
            ulong dj = d_[j] + im;
//            if ( (dj>m1_[j]) || ((long)dj<0) )
            if ( dj>m1_[j] )
            {
                i_[j] = -ij;
            }
            else
            {
                d_[j] = dj;
                dm_ = im;
                j_ = j;
                return 1;
            }

            ++j;
        }
        return 0;
    }

    bool prev()
    {
        // decrement mixed-radix number:
        ulong j = 0;
        ulong ij;
        ulong im;
//        while ( j<n_-1 )
        while ( (ij=i_[j]) )
        {
            im = -i_[j];
            ulong dj = d_[j] + im;
//            if ( (dj>m1_[j]) || ((long)dj<0) )
            if ( dj>m1_[j] )
            {
                i_[j] = -ij;
            }
            else
            {
                d_[j] = dj;
                dm_ = im;
                j_ = j;
                return 1;
            }

            ++j;
        }
        return 0;
    }

    ulong pos()  const  { return j_; }  // position of last change
    int dir()  const  { return dm_; }
};
// -------------------------


#endif  // !defined HAVE_MIXEDRADIX_GRAY2_H__
