#if !defined HAVE_MIXEDRADIX_MODULAR_GRAY_H__
#define      HAVE_MIXEDRADIX_MODULAR_GRAY_H__

#include "fxttypes.h"


class mixedradix_modular_gray
// Gray code for mixed radix numbers
// Implementation following Knuth
{
public:
    ulong *a_;
    ulong *m1_;
    ulong *f_;
    ulong *s_;
    ulong n_;
    ulong j_;


public:
    mixedradix_modular_gray(const ulong *m, ulong n, ulong mm=0)
        : n_(n)
    {
        a_ = new ulong[4*n_+1];
        m1_ = a_ + n_;
        s_ = m1_ + n_;  // m1_[j] == m[j] - 1 in Knuth
        f_ = s_ + n_;   // n_ + 1 elements

        // if mm given it is used as radix for all digits:
        if ( 0==mm )  for (ulong k=0; k<n_; ++k)  m1_[k] = m[k] - 1;
        else          for (ulong k=0; k<n_; ++k)  m1_[k] = mm - 1;

        first();
    }

    ~mixedradix_modular_gray()  { delete [] a_; }

    const ulong *data()  const  { return a_; }


    void first()
    {
        for (ulong k=0; k<n_; ++k)  a_[k] = 0;
        for (ulong k=0; k<n_; ++k)  s_[k] = m1_[k];
        for (ulong k=0; k<=n_; ++k)  f_[k] = k;
        j_ = n_;
    }

    bool next()
    {
        j_ = f_[0];
        f_[0] = 0;

        if ( j_>=n_ )  { first(); return 0; }

        ++a_[j_];  if ( a_[j_] > m1_[j_] )  a_[j_] = 0;

        ulong aj = a_[j_];
        if ( s_[j_]==aj )
        {
            --s_[j_];  if ( s_[j_] == -1UL )  s_[j_] = m1_[j_];

            f_[j_] = f_[j_+1];
            f_[j_+1] = j_ + 1;
        }

        return 1;
    }

    ulong pos()  const  { return j_; }  // position of last change
};
// -------------------------


#endif  // !defined HAVE_MIXEDRADIX_MODULAR_GRAY_H__
