#if !defined HAVE_MIXEDRADIX_H__
#define      HAVE_MIXEDRADIX_H__

#include "fxttypes.h"


class mixedradix
// Mixed radix counting.
{
public:
    ulong *d_;  // digits
    ulong *m1_; // radix (minus one) for each digit
    ulong n_;   // Number of digits
    ulong j_;  // aux pointer

public:
    mixedradix(const ulong *m, ulong n, ulong mm=0)
    {
        n_ = n;
        d_ = new ulong[n_+1];
        d_[n_] = -1UL;  // sentinel
        m1_ = new ulong[n_];

        // if mm given it is used as radix for all digits:
        if ( 0==mm )  for (ulong k=0; k<n_; ++k)  m1_[k] = m[k] - 1;
        else          for (ulong k=0; k<n_; ++k)  m1_[k] = mm - 1;

        first();
    }

    ~mixedradix()
    {
        delete [] d_;
        delete [] m1_;
    }

    const ulong *data()  const  { return d_; }

    void first()
    {
        for (ulong k=0; k<n_; ++k)  d_[k] = 0;
        j_ = n_;
    }

    void last()
    {
        for (ulong k=0; k<n_; ++k)  d_[k] = m1_[k];
        j_ = n_;
    }


    bool next()  // increment
    {
        bool ret = false;
        ulong j = 0;
        while ( d_[j]==m1_[j] )  { d_[j]=0; ++j; }
        if ( j<n_ ) // only if no overflow
        {
            ++d_[j];
            ret = true;
        }
        j_ = j;
        return ret;
    }

    bool prev()  // decrement
    {
        bool ret = false;
        ulong j = 0;
        while ( d_[j]==0 )  { d_[j]=m1_[j]; ++j; }
        if ( j<n_ ) // only if no overflow
        {
            --d_[j];
            ret = true;
        }
        j_ = j;
        return ret;
    }

    ulong pos()  const  { return j_; }  // position of last change
};
// -------------------------


#endif  // !defined HAVE_MIXEDRADIX_H__
