#if !defined  HAVE_PERMGRAY2_H__
#define       HAVE_PERMGRAY2_H__


#include "aux0/swap.h"
#include "fxttypes.h"

class perm_gray2
// Minimal change permutations.
// CAT algorithm based on mixed radix Gray code
//   for the factorial number system.
{
public:
    ulong *d_;  // mixed radix digits with radix = [n-1, n-2, ..., 2]
    ulong *i_;  // direction
    ulong *x_;  // permutation
    ulong *ix_;  // inverse permutation
    ulong n_;   // permutations of n elements
    ulong sw1_, sw2_; // indices of elements swapped most recently

public:
    perm_gray2(ulong n)
    {
        n_ = n;
        d_ = new ulong[n_];
        d_[n-1] = -1UL;  // sentinel
        i_ = new ulong[n_];
        x_ = new ulong[n_];
        ix_ = new ulong[n_];
        i_[n_-1] = 0;
        first();
    }

    ~perm_gray2()
    {
        delete [] i_;
        delete [] d_;
        delete [] x_;
        delete [] ix_;
    }

    const ulong *data()  const { return x_; }
    const ulong *invdata()  const { return ix_; }
    void get_swap(ulong &s1, ulong &s2)  const  { s1=sw1_; s2=sw2_; }


    void first()
    {
        for (ulong k=0; k<n_-1; ++k)  d_[k] = 0;
        for (ulong k=0; k<n_-1; ++k)  i_[k] = +1;
        for (ulong k=0; k<n_; ++k)  x_[k] = ix_[k] = k;
        sw1_=n_-1;  sw2_=n_-2;
    }

    void last()
    {
        first();
        d_[n_-2] = 1;
        swap2(x_[n_-1], x_[n_-2]);
        swap2(ix_[n_-1], ix_[n_-2]);
        for (ulong k=0; k<n_-2; ++k)  i_[k] = -1UL;
        i_[n_-2] = +1;
    }

    bool next()
    {
        // increment mixed-radix number:
        ulong j = 0;
        ulong m1 = n_ - 1;
        ulong ij;
        ulong im;
//        while ( j<n_-1 )
        while ( (ij=i_[j]) )
        {
            im = i_[j];
            ulong dj = d_[j] + im;
//            if ( (dj>m1) || ((long)dj<0) )
            if ( dj>m1 )
            {
                i_[j] = -ij;
            }
            else
            {
                d_[j] = dj;
                swap(j, im);
                return 1;
            }

            --m1;
            ++j;
        }
        return 0;
    }

    bool prev()
    {
        // decrement mixed-radix number:
        ulong j = 0;
        ulong m1 = n_ - 1;
        ulong ij;
        ulong im;
//        while ( j<n_-1 )
        while ( (ij=i_[j]) )
        {
            im = -i_[j];
            ulong dj = d_[j] + im;
//            if ( (dj>m1) || ((long)dj<0) )
            if ( dj>m1 )
            {
                i_[j] = -ij;
            }
            else
            {
                d_[j] = dj;
                swap(j, im);
                return 1;
            }

            --m1;
            ++j;
        }
        return 0;
    }

private:
    void swap(ulong j, ulong im)
    {
        ulong x1 = j;
        ulong i1 = ix_[x1];
        ulong i2 = i1 + im;
        ulong x2 = x_[i2];
        swap2(ix_[x1], ix_[x2]);
        swap2(x_[i1], x_[i2]);
        sw1_=i1;  sw2_=i2;
    }
};
// -------------------------

#endif  // !defined HAVE_PERMGRAY2_H__
