#if !defined  HAVE_PERMLEX_REC_H__
#define       HAVE_PERMLEX_REC_H__


#include "aux0/swap.h"
#include "fxttypes.h"

class perm_lex_rec
{
public:
    ulong *pp_;  // permutation
    ulong n_;    // number of elements
//    ulong ct_;   // count combinations
//    ulong rct_;  // count recursions (==work)
    void (*visit_)(const perm_lex_rec &);  // function to call with each permutation

public:
    perm_lex_rec(ulong n)
    {
        n_ = n;
        pp_ = new ulong[n_];
    }

    ~perm_lex_rec()
    { delete [] pp_; }

    void init()
    {
//        ct_ = 0;
//        rct_ = 0;
        for (ulong k=0; k<n_; ++k)  pp_[k] = k;
    }

    void generate(void (*visit)(const perm_lex_rec &))
    {
        visit_ = visit;
        init();
        next_rec(0);
    }

    void generate_cyclic(void (*visit)(const perm_lex_rec &))
    {
        visit_ = visit;
        init();
        next_cyclic_rec(0);
    }

private:
    void next_rec(ulong d)
    {
        if ( d==n_-1 )
        {
//            ++ct_;  // count objects
            visit_(*this);
        }
        else
        {
//            ++rct_;  // measure computational work
            for (ulong x=d; x<n_; ++x)  // increasing order
//                for (ulong x=n_-1; (long)x>=(long)d; --x)  // decreasing order
            {
                swap2(pp_[d], pp_[x]);
//                reverse(pp_+d, n_-x);
//                rotate_right(pp_+d, n_-d, 1);
                next_rec(d+1);
//                reverse(pp_+d, n_-x);
                swap2(pp_[d], pp_[x]);
            }
        }
    }

    void next_cyclic_rec(ulong d)
    {
        if ( d==n_-1 )
        {
//            ++ct_;  // count objects
            visit_(*this);
        }
        else
        {
//            ++rct_;  // measure computational work
            for (ulong x=d+1; x<n_; ++x)
            {
                swap2(pp_[d], pp_[x]);
                next_cyclic_rec(d+1);
                swap2(pp_[d], pp_[x]);
            }
        }
    }
};
// -------------------------

#endif  // !defined HAVE_PERMLEX_REC_H__
