#if !defined HAVE_PERMLEX_H__
#define      HAVE_PERMLEX_H__

#include "aux0/swap.h"
#include "fxttypes.h"


class perm_lex
{
protected:
    ulong *p;  // current permutation of {0, 1, ..., n-1}
    ulong n;   // number of elements to permute

public:
    perm_lex(ulong nn)
    {
        n = (nn > 0 ? nn : 1);
        p = new ulong[n];
        first();
    }

    ~perm_lex() { delete [] p; }

    void first()
    {
        for (ulong i=0; i<n; i++)  p[i] = i;
    }

    const ulong *data()  const  { return p; }

    bool next()
    // Based on code by Glenn Rhoads
    // (who ascribes the algorithm to Dijkstra)
    {
        const ulong n1 = n - 1;
        ulong i = n1;
        do
        {
            --i;
            if ( (long)i<0 )  return  0;  // last sequence is falling seq.
        }
        while ( p[i] > p[i+1] );

        ulong j = n1;
        while ( p[i] > p[j] )  --j;

        swap2(p[i], p[j]);

        ulong r = n1;
        ulong s = i + 1;
        while ( r > s )
        {
            swap2(p[r], p[s]);
            --r;
            ++s;
        }

        return  1;
    }
};
// -------------------------



#endif  // !defined HAVE_PERMLEX_H__
