#if !defined HAVE_PERMSTAR_H__
#define      HAVE_PERMSTAR_H__

#include "aux0/swap.h"
#include "fxttypes.h"

// define to also compute inverse:
#define PERM_STAR_WITH_INVERSE

class perm_star
// algorithm of Gideon Ehrlich, as given in Knuth
{
public:
    ulong n_;    // number of elements
    ulong swp_;  // index of element that was swapped (with index 0)
    ulong *a_;   // current permutation
    ulong *b_;   // auxiliary array
    ulong *c_;   // auxiliary array
#ifdef PERM_STAR_WITH_INVERSE
    ulong *ia_;   // inverse permutation
#endif
    ulong ct_; // incremented with each call to next()

public:
    perm_star(ulong n)
    {
        n_ = n;
#ifdef PERM_STAR_WITH_INVERSE
        a_ = new ulong[4*n_+1];
#else
        a_ = new ulong[3*n_+1];
#endif
        b_ = a_ + n_;
        c_ = b_ + n_;
#ifdef PERM_STAR_WITH_INVERSE
        ia_ = c_ + n_ + 1;
#endif
        first();
    }

    ~perm_star()
    {
        delete [] a_;
    }

    void first()
    {
        swp_ = n_ - 1;
        for (ulong k=0; k<n_; ++k)  a_[k] = b_[k] = k;
        for (ulong k=0; k<=n_; ++k)  c_[k] = 0;
#ifdef PERM_STAR_WITH_INVERSE
        for (ulong k=0; k<n_; ++k)  ia_[k] = k;
#endif
    }

    bool next()
    {
        ulong k = 1;
        if ( c_[k] == k )
        {
            do
            {
                c_[k] = 0;
                ++k;
            }
            while ( c_[k] >= k );
        }

        if ( k == n_ )   return 0;
        else
        {
            ++c_[k];

            swp_ = b_[k];
            swap2(a_[0], a_[swp_]);
#ifdef PERM_STAR_WITH_INVERSE
            swap2(ia_[a_[0]], ia_[a_[swp_]]);
#endif

            ulong j = 1;
            --k;

            while ( j < k )  // < 0.18 iterations per call
            {
                swap2(b_[j], b_[k]);
                ++j;
                --k;
            }
        }

        return  1;
    }

    ulong get_swap()  const  { return swp_; }
    void get_swap(ulong &s1, ulong &s2)  const  { s1=0; s2=swp_; }

    ulong current()  const  { return ct_; }

    const ulong *data()  const  { return a_; }
#ifdef PERM_STAR_WITH_INVERSE
    const ulong *invdata()  const  { return ia_; }
#endif

};
// -------------------------



#endif  // !defined HAVE_PERMSTAR_H__
