#if !defined  HAVE_PERMRAND_H__
#define       HAVE_PERMRAND_H__

#include "aux0/swap.h"
#include "fxttypes.h"

#include <cstdlib>  // rand()


template <typename Type>
void random_permute(Type *f, ulong n)
// Randomly permute the elements of f.
// Algorithm given by Helmut Herold, attributed to C.L.Robinson.
{
    for (ulong k=1; k<n; ++k)
    {
        ulong r = (ulong)rand();
        r ^= r>>16;  // avoid using low bits of rand alone
        ulong i = r % (k+1);
        swap2(f[k], f[i]);
    }
}
// -------------------------

inline void
random_permutation(ulong *f, ulong n)
// Create a random permutation.
{
    for (ulong k=0; k<n; ++k)  f[k] = k;
    random_permute(f, n);
}
// -------------------------


template <typename Type>
void random_permute_cyclic(Type *f, ulong n)
// Permute the elements of f by a random cyclic permutation.
// Algorithm by S.Sattolo:
//   swap last with rand(first,...,last-1)
//   swap second last with rand(first,...,second last -1)
//     ...
//   swap second with first
{
    for (ulong k=n-1; k>0; --k)
    {
        ulong r = (ulong)rand();
        r ^= r>>16;  // avoid using low bits of rand alone
        ulong i = r % k;
        swap2(f[k], f[i]);
    }
}
// -------------------------

inline void
random_cyclic_permutation(ulong *f, ulong n)
// Create a random permutation that is cyclic.
{
    for (ulong k=0; k<n; ++k)  f[k] = k;
    random_permute_cyclic(f, n);
}
// -------------------------


#endif  // !defined HAVE_PERMRAND_H__
