#if !defined  HAVE_EQUIVCLASSES_H__
#define       HAVE_EQUIVCLASSES_H__

#include "fxttypes.h"


template <typename Type>
ulong *
equivalence_classes(const Type *s, ulong n, bool (*equiv_q)(Type,Type), ulong *q=0)
// Given an equivalence relation '==' (as function equiv_q())
//  and a set s[] with n elements,
// write into q[k] (0<=k<n) the index j of the
//  first element s[j] so that s[k]==s[j].
// For the complexity C:  n<=C<=n*n
//  C=n*n if each element is in its own class
//  C=n   if all elements are in one class
{
    if ( 0==q )  q = new ulong[n];
    for (ulong k=0; k<n; ++k)  q[k] = k;  // each in own class
    for (ulong k=1; k<n; ++k)
    {
        ulong j = k;
        while ( j-- )
        {
            if ( equiv_q(s[j], s[k]) )
            {
                q[k] = q[j];
                break;
            }
        }
    }
    return q;
}
// -------------------------


#endif  // !defined HAVE_EQUIVCLASSES_H__
