#if !defined HAVE_WAVELETFILTER_H__
#define      HAVE_WAVELETFILTER_H__

#include "aux1/aux1double.h" // norm()
#include "fxttypes.h" // ulong
#include <cmath> // fabs()


class wavelet_filter
{
protected:
    const wavelet_filter & operator =(const wavelet_filter &);
    wavelet_filter(const wavelet_filter &);

public:
    double *wvc_, *awvc_;
    long nwc_;

    wavelet_filter(const double *w)
    {
        long i = 0;
        while ( w[i]!=0 )  ++i;
        nwc_ = i;

        wvc_ = new double[nwc_];
        awvc_ = new double[nwc_];
        for (i=0; i<nwc_; ++i)
        {
            wvc_[i] = w[i];

            if ( !(i&1) )  awvc_[nwc_-1-i] = -wvc_[i];
            else           awvc_[nwc_-1-i] = +wvc_[i];
        }
    }

    ~wavelet_filter()
    {
        delete [] wvc_;
        delete [] awvc_;
    }

    double norm()  const  { return  ::norm(wvc_, nwc_); }

    bool check(double eps=1e-6) const
    {
        double v = norm();
        if ( fabs(v-1.0) > eps )  return false;
        return true;
    }

};
// -------------------------

// wavelet/waveletfilter.cc:
extern const wavelet_filter wf_haar2;
extern const wavelet_filter wf_daub4;
extern const wavelet_filter wf_daub6;
extern const wavelet_filter wf_daub8;
extern const wavelet_filter wf_daub10;
extern const wavelet_filter wf_daub12;
extern const wavelet_filter wf_daub20;
extern const wavelet_filter wf_test;

#define  MINM     (4) // must be 4

// inverse_wavelet() is inverse of wavelet()
// iff OFFSET1==OFFSET2
#define  OFFSET1  (0) // default 0
#define  OFFSET2  (0) // default 0


#endif // !defined HAVE_WAVELETFILTER_H__
